'use strict';

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

exports['default'] = transformObjectExpressionIntoStyleSheetObject;
/*
 * @providesModule transformObjectExpressionIntoStyleSheetObject
 */

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _contextify = require('contextify');

var _contextify2 = _interopRequireDefault(_contextify);

var _extend = require('object-assign');

var _extend2 = _interopRequireDefault(_extend);

var _print$types = require('recast');

var n = _print$types.types.namedTypes;
var isBlank = /^\s*$/;

function transformObjectExpressionIntoStyleSheetObject(expr, context) {
  _assert2['default'](n.ObjectExpression.check(expr), 'must be a object expression');

  context = _contextify2['default'](_extend2['default']({}, context));

  context.evaluate = function (node) {
    return this.run(_print$types.print(node).code);
  };

  var result = {};

  expr.properties.forEach(function (property) {
    processTopLevelProperty(property.key, property.value, result, context);
  });

  context.dispose();

  return result;
}

function processTopLevelProperty(key, value, result, context) {
  var name = keyToName(key);

  _assert2['default'](n.ObjectExpression.check(value), 'top-level value must be a object expression');

  result[name] = {};

  processProperties(value.properties, result[name], context);
}

function processProperties(properties, result, context) {
  properties.forEach(function (property) {
    processProperty(property.key, property.value, result, context);
  });
}

function processProperty(key, value, result, context) {
  var name = keyToName(key);

  if (canEvaluate(value, context)) {
    var val = context.evaluate(value);

    _assert2['default'](typeof val === 'string' || typeof val === 'number', 'value must be a string or number');

    if (typeof val === 'string') {
      _assert2['default'](!isBlank.test(val), 'string value cannot be blank');
    }

    result[name] = val;
  } else if (n.ObjectExpression.check(value)) {
    result[name] = {};

    processProperties(value.properties, result[name], context);
  } else if (n.UnaryExpression.check(value) && value.prefix === true && value.operator === '-') {
    _assert2['default'](n.Literal.check(value.argument), 'invalid unary argument type');

    result[name] = -value.argument.value;
  } else {
    _assert2['default'](false, 'invalid value expression type');
  }
}

function keyToName(key) {
  _assert2['default'](n.Identifier.check(key) || n.Literal.check(key) && typeof key.value === 'string', 'key must be a string or identifier');

  return key.name || key.value;
}

function canEvaluate(_x, _x2) {
  var _left;

  var _again = true;

  _function: while (_again) {
    _again = false;
    var expr = _x,
        context = _x2;

    if (n.Literal.check(expr)) {
      return true;
    } else if (n.Identifier.check(expr) && context.hasOwnProperty(expr.name)) {
      return true;
    } else if (n.MemberExpression.check(expr)) {
      if (!(_left = n.Identifier.check(expr.property))) {
        return _left;
      }

      _x = expr.object;
      _x2 = context;
      _again = true;
      continue _function;
    } else if (n.BinaryExpression.check(expr)) {
      if (!(_left = canEvaluate(expr.left, context))) {
        return _left;
      }

      _x = expr.right;
      _x2 = context;
      _again = true;
      continue _function;
    }

    return false;
  }
}
module.exports = exports['default'];